@@ -1,29 +1,31 @@
 /******************************************************************************
  *                                                                            *
  *           GLISSANDO - GLauber Initial State Simulation AND mOre...         *
- *                           ver. 1.61                                        *
+ *                       ver. 1.95, 6 August 2010                             *
  *                                                                            *
  * Authors: Wojciech Broniowski (Wojciech.Broniowski@ifj.edu.pl)              *
  *          Maciej Rybczynski   (Maciej.Rybczynski@pu.kielce.pl)              *
  *          Piotr Bozek         (Piotr.Bozek@ifj.edu.pl)                      *
  *                                                                            *
  * For the detailed description of the program and further references         *
- * to the description of the model, please, refer to arXiv:0710.5731 [nucl-th]*
+ * to the description of the model, please, refer to                          *
+ *                                                                            *
+ * Computer Physics Communications 180(2009)69, arXiv:0710.5731 [nucl-th]     *
  * accessibile from: http://arxiv.org.abs/0710.5731                           *
  *                                                                            *
+ * Implementation of nuclear correlations as in Phys. Rev. C81(2010)064909    *
+ *                                                                            *
  * Homepage: http://www.pu.kielce.pl/homepages/mryb/GLISSANDO/index.html      *
  *                                                                            *
  * This code can be freely used and redistributed. However, if you decide to  *
- * make modifications to the code, the authors would appreciate notification. *
+ * make modifications, the authors would appreciate notification.             *
  * Any publication or display of results obtained using this code must        *
- * include a reference to arXiv:0710.5731 [nucl-th] or the published          *
- * version of it, when available.                                             *
- *                                                                            *
+ * include a reference to our published paper.                                *
  *****************************************************************************/
 
 //! the units for all dimentionful quantities are fm
 
-float ver=1.61; //! version of the code
+float ver=1.95; //! version of the code
 
 #include <math.h>
 #include <time.h>
@@ -44,14 +46,25 @@ using namespace std;
 
 int main(int argc, char **argv) {
 
+if(argc > 1){
+if(strcmp(argv[1],"-v")==0){cout << "GLISSANDO ver: " << ver << endl; return 0;};
+if(strcmp(argv[1],"-h")==0){cout << "Usage:" << endl 
+                                 << "glissando.exe <input_file> <output_Root_file>" << endl
+                                 << "glissando_files <input_file> <output_Root_file> <nucl_dictribution_A> <nucl_distribution_B>" << endl 
+                                 << "see Comp. Phys. Comm. 180(2009)69, http://arxiv.org.abs/0710.5731, for more details"
+                                 << endl; return 0;};
+	    };
+
 /****************************************************************************************************************/
 //! declarations, initialization, input, etc.
 /****************************************************************************************************************/
 
 //! header
 cout << endl << "*******************************************************" << endl;
-cout << "GLISSANDO ver. " << ver << " ( http://arxiv.org.abs/0710.5731 )" << endl; //! title and reference
-cout << "                tested with ROOT ver. 5.08--5.18" << endl;
+cout << "GLISSANDO ver. " << ver << endl << 
+"Computer Physics Communications 180(2009)69, http://arxiv.org.abs/0710.5731" << endl <<
+"and Phys. Rev. C81(2010)064909 for implementation of the NN correlations" << endl; //! title and reference
+cout << "(tested with ROOT ver. 5.08--5.21)" << endl;
 cout << "********************************************************" << endl;
 
 time_t rawtime; 
@@ -116,7 +129,8 @@ char s1[]={"EVENTS"},
      s31[]={"FULL"},
      s32[]={"FBIN"},
      s33[]={"DOBIN"},
-     s34[]={"GA"};
+     s34[]={"GA"},
+     s35[]={"FILES"};
 
 int GAUSS; //! shape of the wounding profile
 	#if(_gauss_!=1)
@@ -127,10 +141,11 @@ int GAUSS; //! shape of the wounding profile
 	#endif
 
 //! the input file can be passed as the first command-line argument
+FILE *in; 
 TString inpfile("input.dat");
 if (argc>1) inpfile = argv[1];
-FILE *in; 
 in = fopen(inpfile,"rt");
+
 cout << "parameters reset in input file " << inpfile << " :" << endl;
 
 //! scan the input file for the parameters reset from the default values
@@ -172,11 +187,20 @@ while (!feof(in)){
   if (!strcmp(t,s31)) {FULL=int(v);goto dalej;}   // parameter controling generation of the full event tree 
   if (!strcmp(t,s32)) {FBIN=int(v);goto dalej;}   // number of histogram bins in the the phi angle
   if (!strcmp(t,s33)) {DOBIN=int(v);goto dalej;}  // if 1, generate binary collision also for the pure wounded-nucleon model
-  if (!strcmp(t,s34)) {GA=v;goto dalej;}          // value of the gaussiab wounding profile at the origin
+  if (!strcmp(t,s34)) {GA=v;goto dalej;}          // value of the gaussian wounding profile at the origin
+  if (!strcmp(t,s35)) {FILES=int(v);goto dalej;}  // not used
  dalej:;
 }
 
+#if(_files_)
+FILES=1;
+#endif
+#if(_files_!=1)
+FILES=0;
+#endif
+
 //! correct wrong input
+ if ((FILES==1) && CD*CD > 0.000001){CD=0; cout << "correction: CD=" << CD << endl;};
  if ((MODEL!=0) && (MODEL!=1) && (MODEL!=2)) {MODEL=0; cout << "correction: MODEL=" << MODEL << endl;};
  if (BMIN<0) {BMIN=0; cout << "correction: BMIN=" << BMIN << endl;};
  if (BMAX<BMIN) {BMAX=BMIN; cout << "correction: BMAX=" << BMAX << endl;};
@@ -227,7 +251,8 @@ UInt_t ISEED1 = ISEED;
 UInt_t ISEED = raa.GetSeed();
 
 //! definition of various counters
-counter2  estd,   //! epsilon standard (fixed-axes) = <r^2 cos(2 phi)>/<r^2>
+counter2  sizec,  //! size <rs>
+estd,		  //! epsilon standard (fixed-axes) = <r^2 cos(2 phi)>/<r^2>
 epart,            //! epsilon participant (variable-axes) = <r^2 cos(2 phi*)>/<r^2>
 estd3,            //! <r^2 cos(3 phi)>/<r^2>
 epart3,           //! <r^2 cos(3 phi*)>/<r^2>
@@ -246,6 +271,7 @@ c2s2,             //! r^2 sin(2 phi)
 c2c3,             //! r^2 cos(3 phi)
 c2s3, c2c4, c2s4, c2c5, c2s5, c2c6, c2s6,         // etc.
 //! rs^2 = xs^2 +ys^2, xs and ys and in the frame of the center-of-mass of the fireball
+size,		  //! rs
 c2c0r,            //! rs^2 for shifted coordinates
 c2c2r,            //! rs^2 cos(2 (phi-phi*))
 c2s2r,            //! rs^2 sin(2 (phi-phi*))
@@ -294,11 +320,13 @@ rhotspot,  //! hot-spots
 rpa,       //! source RDS
 r,         //! radius
 r2,        //! radius squared 
+rco,       //! distance between 2 nucleons
 mm,        //! distance between nucleons squared 
 //
 // Not all moment listed below are computed. If they are of interest to the user, he should uncomment 
 // approprate lines in following parts of the code.
 //
+sizeav,	   //! <rs>
 es,        //! epsilon standard (fixed-axes), <r^2 cos(2 phi)>/<r^2>
 ess,       //! <r^2 sin(2 phi)>/<r^2>
 ep,        //! epsilon participant (variable-axes) = <r^2 cos(2 phi*)>/<r^2>
@@ -311,24 +339,27 @@ es4, es4s, ep4, ep4s, es5, es5s, ep5, ep5s, es6, es6s, ep6, ep6s, // etc.
 //
 ww,        //! source RDS
 phi,       //! polar angle relative to the y-axes
-phirot,    //! roration angle phi*
+phirot,    //! rotation angle phi*
+phirot3,    //! rotation angle maximizing the third moment 
 xx,        //! x center-of-mass coordinate
 yy,        //! y center-of-mass coordinate
 xs,        //! x shifted by xx
 ys,        //! y shifted by yy
 phis,      //! polar angle for the shifted coordinates
 r2s,       //! xs^2+ys^2
+sr2s,      //! sqrt(r2s)
 wd,        //! superimposed weight, generated by the superposition distribution 
 xeps, xseps, xepp, xsepp;
 
 
-//! the Root output file with the results can be passed as the second command-line argument
+//! the ROOT output file with the results can be passed as the second command-line argument
 TFile *fout;
 TString outfile("glissando.root");
 if (argc>2) outfile = argv[2];
-cout << endl << "generates output file " << outfile;
 fout = new TFile(outfile,"RECREATE"); fout->cd();
 
+cout << endl << "generates Root output file " << outfile;
+
 //! tree for storing the parameters of the run
 TTree *param;
 param = new TTree("param","param tree");
@@ -366,6 +397,7 @@ param = new TTree("param","param tree");
         param->Branch("DBIN",&DBIN,"DBIN/F");
         param->Branch("GAUSS",&GAUSS,"GAUSS/I");
         param->Branch("GA",&GA,"GA/F");
+        param->Branch("FILES",&FILES,"FILES/I");
         param->Branch("ver",&ver,"ver/F");
 
         param->Fill(); //! fill the parameter tree
@@ -396,10 +428,11 @@ full_event = new TTree("full_event","full event");
         tree->Branch("b",&b,"b");          // impact parameter
 
 //! fixed-axes and variable-axes cos and sin moments (some commented out)
+	tree->Branch("size",&sizeav,"size"); // <rs>
         tree->Branch("es",&es,"es");       // < r^2 cos(2 phi) >
         tree->Branch("ep",&ep,"ep");       // < r^2 cos(2 (phi-phi*)) >
-//        tree->Branch("es3",&es3,"es3");  // < r^2 cos(3 phi) >
-//        tree->Branch("ep3",&ep3,"ep3");  // < r^2 cos(3 (phi-phi*)) >
+        tree->Branch("es3",&es3,"es3");  // < r^2 cos(3 phi) >
+        tree->Branch("ep3",&ep3,"ep3");  // < r^2 cos(3 (phi-phi*)) >
         tree->Branch("es4",&es4,"es4");    // < r^2 cos(4 phi) >
         tree->Branch("ep4",&ep4,"ep4");    // < r^2 cos(4 (phi-phi*)) >
 //        tree->Branch("es5",&es5,"es5");  // < r^2 cos(5 phi) >
@@ -420,11 +453,15 @@ full_event = new TTree("full_event","full event");
 //        tree->Branch("ep6s",&ep6s,"ep6s"); // < r^2 sin(6 (phi-phi*)) >
 
         tree->Branch("phir",&phirot,"phir"); //! the rotationa angle phi*
+        tree->Branch("phir3",&phirot3,"phir3"); //! the rotationa angle phi3*
         tree->Branch("xx",&xx,"x");          //! x c.m. coordinate
         tree->Branch("yy",&yy,"y");          //! y c.m. coordinate
 
         #if(_profile_)
-           tree1->Branch("r",&r,"r"); //! radius in the nuclear profile on nucleus A
+        tree1->Branch("r",&r,"r"); //! radius in the nuclear profile on nucleus A
+//        tree1->Branch("rco",&rco,"rco"); //! NN correlation
+	TH1D *rad = new TH1D("rad", "one-body distribution", 600, 0., 15.);
+	TH1D *rrel = new TH1D("rrel", "relative distance", 800, 0.000001, 4.);
         #endif
 
         #if(_weight_)
@@ -461,18 +498,32 @@ full_event = new TTree("full_event","full event");
 	TH1F *ny = new TH1F("ny", "<y> vs. N_{w}", NUMA+NUMB, 0.5, NUMA+NUMB+0.5);
 	TH1F *ny2= new TH1F("ny2", "var(y) vs. N_{w}", NUMA+NUMB, 0.5, NUMA+NUMB+0.5); 
 
-	TH1F *neps = new TH1F("neps", "#epsilon vs. N_{w}", NUMA+NUMB, 0.5, NUMA+NUMB+0.5);
+	TH1F *nsize = new TH1F("nsize", "<r> vs. N_{w}", NUMA+NUMB, 0.5, NUMA+NUMB+0.5);
+	TH1F *nsize2 = new TH1F("nsize2", "<r>^2 vs. N_{w}", NUMA+NUMB, 0.5, NUMA+NUMB+0.5);
+
+	TH1F *neps = new TH1F("neps", "#epsilon_{2} vs. N_{w}", NUMA+NUMB, 0.5, NUMA+NUMB+0.5);
+	TH1F *neps4 = new TH1F("neps4", "#epsilon_{4} vs. N_{w}", NUMA+NUMB, 0.5, NUMA+NUMB+0.5);
+
 	TH1F *neps2= new TH1F("neps2", "var(#epsilon)/#epsilon^{2} vs. N_{w}",  NUMA+NUMB, 0.5, NUMA+NUMB+0.5); 
-	TH1F *nepsp = new TH1F("nepsp", "#epsilon^{*} vs. N_{w}",  NUMA+NUMB, 0.5, NUMA+NUMB+0.5);
+
+	TH1F *nepsp = new TH1F("nepsp", "#epsilon^{*}_{2} vs. N_{w}",  NUMA+NUMB, 0.5, NUMA+NUMB+0.5);
+	TH1F *nepsp4 = new TH1F("nepsp4", "#epsilon^{*}_{4} vs. N_{w}",  NUMA+NUMB, 0.5, NUMA+NUMB+0.5);
+
 	TH1F *nepsp2= new TH1F("nepsp2", "var(#epsilon*)/#epsilon*^{2} vs. N_{w}",  NUMA+NUMB, 0.5, NUMA+NUMB+0.5); 
 	TH1F *nuni = new TH1F("nuni", "event multiplicity vs. N_{w}",  NUMA+NUMB, 0.5, NUMA+NUMB+0.5);
 
 	TH1F *nepsb = new TH1F("nepsb", "#epsilon vs. b", 200, BMIN, BMAX);
 	TH1F *neps2b= new TH1F("neps2b", "var(#epsilon)/#epsilon^{2} vs. b", 200, BMIN, BMAX); 
+
 	TH1F *nepspb = new TH1F("nepspb", "#epsilon* vs. b", 200, BMIN, BMAX);
 	TH1F *nepsp2b= new TH1F("nepsp2b", "var(#epsilon*)/#epsilon*^{2} vs. b", 200, BMIN, BMAX); 
+
 	TH1F *nunib = new TH1F("nunib", "event multiplicity vs. b", 200, BMIN, BMAX);
 
+	TH1F *nwb = new TH1F("nwb", "N_{w} vs. b", 200, BMIN, BMAX);
+	TH1F *nw2b= new TH1F("nw2b", "var(N_{w}) vs. b", 200, BMIN, BMAX); 
+
+
 //! histograms for fluctuations of number of wounded nucleons and RDS
 	TH1F *nwei = new TH1F("nwei", "RDS vs. N_{w}^{A}",  NUMA, 0.5, NUMA+0.5);
 	TH1F *nwei2= new TH1F("nwei2", "var(RDS)/RDS vs. N_{w}^{A}",  NUMA, 0.5, NUMA+0.5); 
@@ -485,7 +536,7 @@ full_event = new TTree("full_event","full event");
 
 float CD2=CD*CD;
 d=sqrt(SNN/PI/10.);      //! the wounding distance, 10 converts to fm^2 from mb
-float d2=d*2;
+float d2=d*d;
 dbin=d;                  //! the binary collision distance (= the wounding distance)
 float dbin2=dbin*dbin;
 float mbin=SBIN/SNN;     //! ratio for the hot-spot model
@@ -500,10 +551,10 @@ cout << "seed: " << raa.GetSeed()  << ", number of events: " << EVENTS << endl;
    cout << "generates the nuclear density profile for nucleus A" << endl;
 #endif
 cout << NUMA <<"+" << NUMB;
-if(NUMA>2){cout << ", RA="  << RWSA << "fm, aA="<< AWSA << "fm";};
-cout << ", RB=" << RWSB << "fm, aB=" << AWSB << "fm, d=" << CD  << "fm";
-if(WFA*WFA>0.00000001) {cout << ", wA="<< WFA;}
-if(WFB*WFB>0.00000001) {cout << ", wB="<< WFB;}
+if((NUMA>2) && (FILES!=1)){cout << ", RA="  << RWSA << "fm, aA="<< AWSA << "fm";};
+if(FILES!=1){cout << ", RB=" << RWSB << "fm, aB=" << AWSB << "fm, d=" << CD  << "fm";};
+if((WFA*WFA>0.00000001) && (FILES!=1)) {cout << ", wA="<< WFA;}
+if((WFB*WFB>0.00000001) && (FILES!=1)) {cout << ", wB="<< WFB;}
 cout << endl; 
 if(ALPHA==0){cout << "wounded nucleon model: sig_w=" << SNN << "mb" << endl;};
 if(ALPHA==0 && DOBIN==1){cout << "   (binary collisions counted)" << endl;};
@@ -533,11 +584,61 @@ if(DW>0.0 || DBIN >0.0) {cout << "source dispersion parameter: wounded=" << DW <
 cout << endl; 
 if(FULL){cout << "full event tree generated (THIS GENERATES A LOT OF DATA, set FULL=0 to switch off)" << endl;};
 
+/****************************************************************************************************************/
+
+
+#if(_files_) //! initialize tables for the nucleon distributions
+
+//! the input file can be passed as the third and fourth command-line arguments
+FILE *nucleusA, *nucleusB;
+
+TString nAfile("pb208.dat");
+if (argc>3) nAfile = argv[3];
+nucleusA = fopen(nAfile,"rt");
+
+TString nBfile("pb208.dat");
+if (argc>4) nBfile = argv[4];
+nucleusB = fopen(nBfile,"rt");
+
+cout  << "reading data for nucleus A from " << nAfile << endl;
+cout  << "reading data for nucleus B from " << nBfile << endl << endl;
+
+int countA=0, countB=0;
+
+//! maximum number of nuclei in each file
+const int snu=100000;
+
+//! length of array to store the data
+int snuA=snu*NUMA, snuB=snu*NUMB;
+
+int charge;
+
+float* posAx=NULL; float* posBx=NULL; float* posAy=NULL; float* posBy=NULL; float* posAz=NULL; float* posBz=NULL;
+
+posAx=new float[snuA]; posBx=new float[snuB];
+posAy=new float[snuA]; posBy=new float[snuB];
+posAz=new float[snuA]; posBz=new float[snuB];
+
+for(i=0;i<snuA;i++){
+                fscanf(nucleusA,"%f %f %f %d",&x,&y,&z,&charge);
+		posAx[i]=x;
+	        posAy[i]=y;
+	        posAz[i]=z;};
+
+for(i=0;i<snuB;i++){
+                fscanf(nucleusB,"%f %f %f %d",&x,&y,&z,&charge);
+		posBx[i]=x;
+	        posBy[i]=y;
+	        posBz[i]=z;};
+
+#endif
+
 /****************************************************************************************************************/
 //! simulation
 /****************************************************************************************************************/
 
 //! loop over events
+
 kk=0; //! counter of events where a collision occurs
 for(k=1;kk<EVENTS;k++){
 
@@ -546,6 +647,14 @@ evall++; //! attempted A+B collision (all cases, those where an NN collision occ
 //! generation of b with the distribution proportional to b^2 on the range (BMAX, BMIN)
 b=sqrt((BMAX*BMAX-BMIN*BMIN)*los()+BMIN*BMIN);
 
+#if(_files_)
+//! set random nuclei A nad B, set the position in the data tables appropriately
+countA=los()*(snu-1);
+countA=countA*NUMA;
+countB=los()*(snu-1);
+countB=countB*NUMB;
+#endif
+
 nbin=0; nwA=0; nwB=0; nhotspot=0;  //! reset the counters of binary collisions, wounded nucleons, and hot-spots
 
 xc.reset();yc.reset();zc.reset(); //! reset the center-of-mass counter of nucleus A
@@ -561,9 +670,17 @@ if(RET==1){ //! return-to-beginning algorithm
 		#if(_profile_)
 		   zc.reset();
 		#endif
+		#if(_files_!=1)
 		spgenA(&x,&y,&z); //! generate the first nucleon
-		nA[0][0]=x;
-		nA[0][1]=y;
+		#endif
+		#if(_files_)
+		x=posAx[countA];
+		y=posAy[countA];
+		z=posAz[countA];
+                countA++;
+		#endif
+ 		nA[0][0]=x;
+ 		nA[0][1]=y;
 		nA[0][2]=z;
 		xc.add(x,1);
 		yc.add(y,1);
@@ -572,7 +689,15 @@ if(RET==1){ //! return-to-beginning algorithm
 		#endif
 		good=1;
 		while(good){
-			spgenA(&x,&y,&z); //! generate subsequent nucleons
+		#if(_files_!=1)
+		spgenA(&x,&y,&z); //! generate subsequent nucleons
+		#endif
+		#if(_files_)
+		x=posAx[countA];
+		y=posAy[countA];
+		z=posAz[countA];
+		countA++;
+		#endif
 			for(j=1;(j<fA) && good;j++){
 				mm=(x-nA[j-1][0])*(x-nA[j-1][0])+(y-nA[j-1][1])*(y-nA[j-1][1])+(z-nA[j-1][2])*(z-nA[j-1][2]);
   				if(mm<CD2){good=0;} //! if closer than the expulsion distance, break
@@ -591,7 +716,15 @@ if(RET==1){ //! return-to-beginning algorithm
 	}
 else { //! fix-last algorithm 
 	fA=1;
+	#if(_files_!=1)
 	spgenA(&x,&y,&z); //! generate the first nucleon
+	#endif
+	#if(_files_)
+	x=posAx[countA];
+	y=posAy[countA];
+	z=posAz[countA];
+	countA++; 
+	#endif
 	nA[0][0]=x;
 	nA[0][1]=y;
 	nA[0][2]=z;
@@ -599,7 +732,20 @@ else { //! fix-last algorithm
 	yc.add(y,1);
 	zc.add(z,1); 
 	while(fA<NUMA){
-		spgenA(&x,&y,&z); //! generate subsequent nucleons
+	#if(_files_!=1)	
+	spgenA(&x,&y,&z); //! generate subsequent nucleons
+	#endif
+	#if(_files_)
+		x=posAx[countA];
+		y=posAy[countA];
+		z=posAz[countA];
+// take a random nucleon to avoid correlations
+//		int ccran=los()*(snu-1)*NUMA;
+//		x=posAx[ccran];
+//		y=posAy[ccran];
+//		z=posAz[ccran];
+		countA++; 
+		#endif
 		good=1;
 		for(j=1;j<fA;j++){
 			mm=(x-nA[j-1][0])*(x-nA[j-1][0])+(y-nA[j-1][1])*(y-nA[j-1][1])+(z-nA[j-1][2])*(z-nA[j-1][2]);
@@ -621,11 +767,22 @@ else { //! fix-last algorithm
 
 #if(_profile_) //! store the nuclear density profile
 for(j=1;j<=fA;j++){
-	x=nA[j-1][0]-xc.mean();
-	y=nA[j-1][1]-yc.mean();
-	z=nA[j-1][2]-zc.mean();
+	x=nA[j-1][0]-0*xc.mean();
+	y=nA[j-1][1]-0*yc.mean();
+	z=nA[j-1][2]-0*zc.mean();
 	r=sqrt(x*x+y*y+z*z); 
-	tree1->Fill();};
+	tree1->Fill();
+        rad->Fill(r,1.);
+	};
+
+for(j=1;j<=fA;j++){for(i=1;i<j;i++){
+	x=nA[j-1][0]-nA[i-1][0];
+	y=nA[j-1][1]-nA[i-1][1];
+	z=nA[j-1][2]-nA[i-1][2];
+	rco=sqrt(x*x+y*y+z*z); 
+//	tree1->Fill();
+	rrel->Fill(rco,1.);
+	};};
 #endif
 
 #endif //! nucleus A
@@ -682,7 +839,15 @@ if(RET==1){ //! return-to-beginnng algorithm
 		fA=1;
 		xc.reset();
 		yc.reset();
+		#if(_files_!=1)
 		spgenB(&x,&y,&z);
+		#endif
+		#if(_files_)
+		x=posBx[countB];
+		y=posBy[countB];
+		z=posBz[countB];
+		countB++;
+		#endif
 		nB[0][0]=x;
 		nB[0][1]=y;
 		nB[0][2]=z;
@@ -690,7 +855,15 @@ if(RET==1){ //! return-to-beginnng algorithm
 		yc.add(y,1);
 		good=1;
 		while(good){
-			spgenB(&x,&y,&z);
+		#if(_files_!=1)
+		spgenB(&x,&y,&z);
+		#endif
+		#if(_files_)
+		x=posBx[countB];
+		y=posBy[countB];
+		z=posBz[countB];
+		countB++;
+		#endif
 			for(j=1;(j<fA) && good;j++){
 				mm=(x-nB[j-1][0])*(x-nB[j-1][0])+(y-nB[j-1][1])*(y-nB[j-1][1])+(z-nB[j-1][2])*(z-nB[j-1][2]);
   				if(mm<CD2){good=0;}
@@ -706,14 +879,30 @@ if(RET==1){ //! return-to-beginnng algorithm
 }
 else { //! fix-last algorithm 
 	fA=1;
+	#if(_files_!=1)
 	spgenB(&x,&y,&z);
+	#endif
+	#if(_files_)
+	x=posBx[countB];
+	y=posBy[countB];
+	z=posBz[countB];
+	countB++;
+	#endif
 	nB[0][0]=x;
 	nB[0][1]=y;
 	nB[0][2]=z;
 	xc.add(x,1);
 	yc.add(y,1);  
 	while(fA<NUMB){
+		#if(_files_!=1)
 		spgenB(&x,&y,&z);
+		#endif
+		#if(_files_)
+		x=posBx[countB];
+		y=posBy[countB];
+		z=posBz[countB];
+		countB++;
+		#endif
 		good=1;
 		for(j=1;j<fA;j++){
 			mm=(x-nB[j-1][0])*(x-nB[j-1][0])+(y-nB[j-1][1])*(y-nB[j-1][1])+(z-nB[j-1][2])*(z-nB[j-1][2]);
@@ -769,7 +958,7 @@ for(i=1;i<=NUMA;i++){
  			wa[wc-1]=(1-ALPHA)/2.0*dist(MODEL,Uw); //! wounded nucleon RDS
 			#if(_weight_)
 			   wd=wa[wc-1]/(1-ALPHA)*2.0; //! the weight of the superimposed distribution
-			   tree1->Fill();
+		//	   tree1->Fill();
 			#endif
            		rpa=rpa+wa[wc-1];
 	        	if(wa[wc-1]>0.0){
@@ -873,8 +1062,12 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
 
 	//! compute the fixed-axes moments needed for determination of the rotation angle
         c2c2.reset(); 
-	c2s2.reset(); 
+	c2s2.reset();
+        c2c3.reset(); 
+	c2s3.reset();
+ 
 	for(j=1;j<=wc+nbin;j++){
+
 		x=wx[j-1]-xx;
 		y=wy[j-1]-yy; //shifted coordinates
 		phi=atanm(x,y); // use the corrected arctan
@@ -883,22 +1076,32 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
        		if(r2>0.0){
 			c2c2.add(r2*cos(2*phi),ww);
 			c2s2.add(r2*sin(2*phi),ww);
+			c2c3.add(r2*cos(3*phi),ww);
+			c2s3.add(r2*sin(3*phi),ww);
 		}
        		else {
 			c2c2.add(0.0,ww);
 			c2s2.add(0.0,ww);
+			c2c3.add(0.0,ww);
+			c2s3.add(0.0,ww);
 		};
 	} //for
 
 	//! determination of the rotation angle which maximizes the second harmonic of the distribution
-	phirot=atan(c2s2.mean()/c2c2.mean())/2;
-        if(c2c2.mean()/cos(2*phirot)<0){phirot=phirot-PI/2;};
-        if(phirot<-PI/2){phirot=phirot+2*PI/2;};
+	phirot=atan2(c2s2.mean(),c2c2.mean())/2;
+//        if(c2c2.mean()/cos(2*phirot)<0){phirot=phirot-PI/2;};
+//        if(phirot<-PI/2){phirot=phirot+2*PI/2;};
+
+	//! determination of the rotation angle which maximizes the third harmonic of the distribution
+	phirot3=atan2(c2s3.mean(),c2c3.mean())/3;
+ //       if(c2c3.mean()/cos(2*phirot3)<0){phirot3=phirot3-PI/2;};
+ //       if(phirot3<-PI/2){phirot3=phirot3+2*PI/2;};
+
 
 	//! compute the fixed-axes and variable axes moments
         c2c0.reset(); c2c2.reset(); c2c3.reset(); c2c4.reset(); c2c5.reset(); c2c6.reset(); 
         c2s2.reset(); c2s3.reset(); c2s4.reset(); c2s5.reset(); c2s6.reset();
-	c2c0r.reset(); c2c2r.reset(); c2c3r.reset(); c2c4r.reset(); c2c5r.reset(); c2c6r.reset();
+	size.reset(); c2c0r.reset(); c2c2r.reset(); c2c3r.reset(); c2c4r.reset(); c2c5r.reset(); c2c6r.reset();
 	c2s2r.reset(); c2s3r.reset(); c2c4r.reset(); c2s5r.reset(); c2s6r.reset();
 
 	//! if SHIFT=1 shift the coordinates x and y to the center-of-mass of the fireball, if SHIFT=0, do not
@@ -913,6 +1116,7 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
 		ys=wy[j-1]-yc.mean();
 		phis=atanm(xs,ys); // use the corrected arctan
        		r2s=xs*xs+ys*ys;
+		sr2s=sqrt(r2s);
         	ww=wa[j-1]; //! the RDS
 
 		//! write the full info on the source: x and y coordinates, RDS, and event number 
@@ -929,7 +1133,7 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
           		c2c0.add(r2,ww);
           		c2c2.add(r2*cos(2*phi),ww);
 			//  c2s2.add(r2*sin(2*phi),ww);
-			//  c2c3.add(r2*cos(3*phi),ww);
+			c2c3.add(r2*cos(3*phi),ww);
           		c2s3.add(r2*sin(3*phi),ww);
           		c2c4.add(r2*cos(4*phi),ww);
 			//  c2s4.add(r2*sin(4*phi),ww);
@@ -942,7 +1146,7 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
           		c2c0.add(0.0,ww);
           		c2c2.add(0.0,ww);
 			//  c2s2.add(0.0,ww);
-			//  c2c3.add(0.0,ww);
+			c2c3.add(0.0,ww);
           		c2s3.add(0.0,ww);
           		c2c4.add(0.0,ww);
 			//  c2s4.add(0.0,ww);
@@ -953,23 +1157,25 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
 		};
 
 		//! compute the variable-axes moments (use rs^2)
-		if(r2s>0){          
+		if(r2s>0){ 
+			size.add(sr2s,ww);         
           		c2c0r.add(r2s,ww);
           		c2c2r.add(r2s*cos(2*(phis-phirot)),ww);
-			//  c2c3r.add(r2s*cos(3*(phis-phirot)),ww);
+			c2c3r.add(r2s*cos(3*(phis-phirot3)),ww);
           		c2c4r.add(r2s*cos(4*(phis-phirot)),ww);
 			//  c2c5r.add(r2s*cos(5*(phis-phirot)),ww);
           		c2c6r.add(r2s*cos(6*(phis-phirot)),ww);
 			//  c2s2r.add(r2s*sin(2*(phis-phirot)),ww);
-          		c2s3r.add(r2s*sin(3*(phis-phirot)),ww);
+          		c2s3r.add(r2s*sin(3*(phis-phirot3)),ww);
 			//  c2s4r.add(r2s*sin(4*(phis-phirot)),ww);
 			//  c2s5r.add(r2s*sin(5*(phis-phirot)),ww);
 			//  c2s6r.add(r2s*sin(6*(phis-phirot)),ww);
 		}
   		else {
-          		c2c0r.add(0.0,ww);
+          		size.add(0.0,ww);
+			c2c0r.add(0.0,ww);
           		c2c2r.add(0.0,ww);
-			//  c2c3r.add(0.0,ww);
+			c2c3r.add(0.0,ww);
           		c2c4r.add(0.0,ww);
 			//  c2c5r.add(0.0,ww);
           		c2c6r.add(0.0,ww);
@@ -996,7 +1202,7 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
 		//! fixed-axes
           	c0hist->Fill(sqrt(r2),phi,wei);
           	c2hist->Fill(sqrt(r2),phi,wei*cos(2*phi));
-		//  c3hist->Fill(sqrt(r2),phi,wei*cos(3*phi));
+		c3hist->Fill(sqrt(r2),phi,wei*cos(3*phi));
           	s3hist->Fill(sqrt(r2),phi,wei*sin(3*phi));
           	c4hist->Fill(sqrt(r2),phi,wei*cos(4*phi));
 		//  c5hist->Fill(sqrt(r2),phi,wei*cos(5*phi));
@@ -1007,10 +1213,15 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
 		if(phirr<-PI) {phirr=phirr+2*PI;};
 		if(phirr>PI) {phirr=phirr-2*PI;};
 
+		Float_t phirr3=phis-phirot3;
+		if(phirr3<-PI) {phirr3=phirr3+2*PI;};
+		if(phirr3>PI) {phirr3=phirr3-2*PI;};
+
+
           	c0rhist->Fill(sqrt(r2s),phirr,weis);
           	c2rhist->Fill(sqrt(r2s),phirr,weis*cos(2*(phis-phirot)));
-		//  c3rhist->Fill(sqrt(r2s),phirr,weis*cos(3*(phis-phirot)));
-          	s3rhist->Fill(sqrt(r2s),phirr,weis*sin(3*(phis-phirot)));
+		c3rhist->Fill(sqrt(r2s),phirr3,weis*cos(3*(phis-phirot3)));
+          	s3rhist->Fill(sqrt(r2s),phirr3,weis*sin(3*(phis-phirot3)));
           	c4rhist->Fill(sqrt(r2s),phirr,weis*cos(4*(phis-phirot)));
 		//  c5rhist->Fill(sqrt(r2s),phirr,weis*cos(5*(phis-phirot)));
           	c6rhist->Fill(sqrt(r2s),phirr,weis*cos(6*(phis-phirot)));
@@ -1020,9 +1231,10 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
         nwounded.add(rwAB);nbinary.add(rbin);nhot.add(rhotspot);nweight.add(rpa);
    
 	//! fixed-axes (es_i) and variable-axes (ep_i) moments
+	sizeav=size.mean();
         es=c2c2.mean()/c2c0.mean();  
 	ep=c2c2r.mean()/c2c0r.mean();
-	//  es3=c2c3.mean()/c2c0.mean();ep3=c2c3r.mean()/c2c0r.mean();
+	es3=c2c3.mean()/c2c0.mean();ep3=c2c3r.mean()/c2c0r.mean();
         es4=c2c4.mean()/c2c0.mean();
 	ep4=c2c4r.mean()/c2c0r.mean();  
 	//  es5=c2c5.mean()/c2c0.mean();ep5=c2c5r.mean()/c2c0r.mean();  
@@ -1037,9 +1249,10 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
 	// es5s=c2s5.mean()/c2c0.mean();ep5s=c2s5r.mean()/c2c0r.mean();  
 	// es6s=c2s6.mean()/c2c0.mean();ep6s=c2s6r.mean()/c2c0r.mean();
 
+	sizec.add(sizeav);
         estd.add(es); 
 	epart.add(ep); 
-	// estd3.add(es3);epart3.add(ep3);
+	estd3.add(es3);epart3.add(ep3);
         estd4.add(es4);
 	epart4.add(ep4);
 	// estd5.add(es5);epart5.add(ep5);
@@ -1052,9 +1265,14 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
         ny -> Fill(rwAB,yy);
         ny2 -> Fill(rwAB,yy*yy);
 
+	nsize ->  Fill(rwAB,sizeav);
+	nsize2 -> Fill(rwAB,sizeav*sizeav);
+
         neps -> Fill(rwAB,es);
+        neps4 -> Fill(rwAB,es4);
         neps2 -> Fill(rwAB,es*es);
         nepsp -> Fill(rwAB,ep);
+        nepsp4 -> Fill(rwAB,ep4);
         nepsp2 -> Fill(rwAB,ep*ep);
         nuni -> Fill(rwAB,1);
 
@@ -1062,6 +1280,10 @@ if((wc>=WMIN) && (wc >= W0) && (wc <= W1) && (rpa >= RDS0) && (rpa <= RDS1)&& (n
         neps2b -> Fill(b,es*es);
         nepspb -> Fill(b,ep);
         nepsp2b -> Fill(b,ep*ep);
+
+	nwb -> Fill(b,rwAB);
+	nw2b -> Fill(b,rwAB*rwAB);
+
         nunib -> Fill(b,1);
 
 //! for multiplicity fluctuations
@@ -1100,7 +1322,7 @@ float sirad=sqrt(sitot/PI/10.)/2.; //! equivalent hard-sphere radius in fm
 //! project out the marginal distribution in the radial variable (the radial Fourier profiles)
 TH1D *c0hp = c0hist->ProjectionX("c0hp");
 TH1D *c2hp = c2hist->ProjectionX("c2hp");
-// TH1D *c3hp = c3hist->ProjectionX("c3hp");
+TH1D *c3hp = c3hist->ProjectionX("c3hp");
 TH1D *s3hp = s3hist->ProjectionX("s3hp");
 TH1D *c4hp = c4hist->ProjectionX("c4hp");        
 // TH1D *c5hp = c5hist->ProjectionX("c5hp");        
@@ -1109,7 +1331,7 @@ TH1D *c6hp = c6hist->ProjectionX("c6hp");
 TH1D *c0rhp = c0rhist->ProjectionX("c0rhp");
 TH1D *c2rhp = c2rhist->ProjectionX("c2rhp");
 TH1D *s3rhp = s3rhist->ProjectionX("s3rhp");
-// TH1D *c3rhp = c3rhist->ProjectionX("c3rhp");        
+TH1D *c3rhp = c3rhist->ProjectionX("c3rhp");        
 TH1D *c4rhp = c4rhist->ProjectionX("c4rhp");
 // TH1D *c5rhp = c5rhist->ProjectionX("c5rhp");        
 TH1D *c6rhp = c6rhist->ProjectionX("c6rhp");
@@ -1119,14 +1341,14 @@ TH1D *c6rhp = c6rhist->ProjectionX("c6rhp");
 #endif
 c0hp->Write(); 
 c2hp->Write(); 
-//c3hp->Write(); 
+c3hp->Write(); 
 s3hp->Write();
 c4hp->Write(); 
 //c5hp->Write(); 
 c6hp->Write(); 
 c0rhp->Write();
 c2rhp->Write(); 
-//c3rhp->Write();
+c3rhp->Write();
 s3rhp->Write(); 
 c4rhp->Write(); 
 //c5rhp->Write();
@@ -1151,8 +1373,19 @@ c0rhist->Write();
         ny->Multiply(ny,ny);
         ny2->Add(ny,-1);
 
+	nsize -> Divide(nsize,nuni);
+	nsize -> Write();
+	nsize2-> Divide(nsize2,nuni);
+	nsize -> Multiply(nsize,nsize);
+	nsize2-> Add(nsize,-1);
+	nsize2-> Divide(nsize2,nsize);
+	nsize2-> Write();
+
         neps->Divide(neps,nuni);
         neps->Write();
+        neps4->Divide(neps4,nuni);
+        neps4->Write();
+
         neps2->Divide(neps2,nuni);
         neps->Multiply(neps,neps);
         neps2->Add(neps,-1);
@@ -1160,6 +1393,9 @@ c0rhist->Write();
 
         nepsp->Divide(nepsp,nuni);
         nepsp->Write();
+        nepsp4->Divide(nepsp4,nuni);
+        nepsp4->Write();
+
         nepsp2->Divide(nepsp2,nuni);
         nepsp->Multiply(nepsp,nepsp);
         nepsp2->Add(nepsp,-1);
@@ -1179,6 +1415,12 @@ c0rhist->Write();
         nepsp2b->Add(nepspb,-1);
         nepsp2b->Divide(nepsp2b,nepspb);
 
+	nwb -> Divide(nwb,nunib);
+	nwb -> Write();
+	nw2b -> Divide(nw2b,nunib);
+	nwb -> Multiply(nwb,nwb);
+	nw2b -> Add(nwb,-1);
+
         nx2->Write();
         ny2->Write();
         neps2->Write();
@@ -1186,6 +1428,7 @@ c0rhist->Write();
         nuni->Write();
         neps2b->Write();
         nepsp2b->Write();
+	nw2b->Write();
         nunib->Write();
 
 	//! for multiplicity fluctuations
@@ -1225,6 +1468,11 @@ c0rhist->Write();
       	param->Write();
       	tree->Write();
       	phys->Write(); 
+ 	#if(_profile_)
+	  rad->Write();
+	  rrel->Write();
+	#endif
+
       	if(FULL){full_event->Write();}; //! write the full event info
 
 //! output  basic results to the console
@@ -1239,9 +1487,9 @@ c0rhist->Write();
       cout << "relative deposited strength (RDS) = " << nweight.mean()  << "+/-" << sqrt(nweight.var()) << endl; 
       cout << "eps_fix_2  (std.) = " << estd.mean() << "+/-" << sqrt(estd.var()) << endl;
       cout << "eps_var_2 (part.) = " << epart.mean() << "+/-" << sqrt(epart.var()) << endl;
-      cout << "eps_fix_4 = " << estd4.mean() << "+/-" << sqrt(estd4.var()) << endl;      
-      cout << "eps_var_4 = " << epart4.mean() << "+/-" << sqrt(epart4.var()) << endl;
-      cout << "x_cm = " << xxc.mean() << "+/-" << sqrt(xxc.var()) << ",  y_cm = " << yyc.mean() << "+/-" << sqrt(yyc.var()) << endl;
+//      cout << "eps_fix_4 = " << estd4.mean() << "+/-" << sqrt(estd4.var()) << endl;      
+//      cout << "eps_var_4 = " << epart4.mean() << "+/-" << sqrt(epart4.var()) << endl;
+//      cout << "x_cm = " << xxc.mean() << "+/-" << sqrt(xxc.var()) << ",  y_cm = " << yyc.mean() << "+/-" << sqrt(yyc.var()) << endl;
 
 //! footer
 cout << endl; time ( &rawtime ); timeinfo = localtime ( &rawtime ); 
